#pragma OPENCL EXTENSION cl_khr_global_int32_base_atomics : enable
#pragma OPENCL EXTENSION cl_khr_global_int32_extended_atomics : enable

typedef unsigned char uint8_t;
typedef unsigned int uint32_t;
typedef unsigned long uint64_t;

#define PARAM_BIG_N (1 << 21)
#define NUM_OF_ITER (PARAM_BIG_N / 2)

typedef struct _THE_PAIR
{
	uint32_t Index1;
	uint32_t Index2;
} THE_PAIR;

__kernel void Shader(__global uint32_t Data[PARAM_BIG_N], __global THE_PAIR Result[PARAM_BIG_N], __global uint32_t* Size)
{
	size_t GID = get_global_id(0);
	if(GID < PARAM_BIG_N)
	{
		if(GID == 0)
		{
			*Size = 0;
		}
		uint32_t MyValue = Data[GID];
		
		for(uint32_t i = 1; i <= NUM_OF_ITER; i++)
		{
			if((i < NUM_OF_ITER) || (GID < NUM_OF_ITER / 2))
			{
				size_t CurOffset = (GID + i) % PARAM_BIG_N;
				if(Data[CurOffset] == MyValue)
				{
					uint32_t NewIndex;
					uint32_t OldIndex = 0;
					
					do
					{
						NewIndex = OldIndex + 1;
						if(NewIndex > NUM_OF_ITER)
						{
							return;
						}
						OldIndex = atomic_cmpxchg(Size, OldIndex, NewIndex);
					} while(OldIndex >= NewIndex);
					
					Result[OldIndex].Index1 = (uint32_t)GID;
					Result[OldIndex].Index2 = ((uint32_t)GID + i) % PARAM_BIG_N;
				}
			}
		}
	}
}